%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% we have also implemented the non-sparse bayesian extreme learning
% machine for multiclassification(BELM).The difference between the SBELM and BELM model is that 
% SBELM adopts indepandent ARD priors for the distribution of output weights
% while BELM just adopts one prior governing all weights distribution. For details, please refer to our
% paper: Sparse Bayesian Extreme Learning Machine by Jiahua Luo and Chi-Man
% Vong, etc.
%The calling of this function is the same as Sbelm_classify. The prediction
%of BELM can directly call the Sbelm_predict(...) 
%  Input argumenets:
%  X_train :                     - N-by-M matrix of training data. 
%                                 N is the number of training
%                               data. M: the dimension of attribute.
%  T_train :                     -observed outputs, recommending labeling as
%                               [0,1,2,...C,]
%  label :                      -labels of all categories.
%  NumberofHiddenNeurons  :     - number of hidden neurons 
%  Activation [optional]  :    - the activation of hidden layer
%  seed [optional]        :    - seed for generating uniformly distributed
%                                synapses for input-to-hidden layer.
%                                different seed may have (+ -)1~2% impact on the testing accuracy. 
% Output arguments: 
%  Model :                - the structure for saving models. contains C*(C-1)/2
%                         classifiers
%  ActiveNodes:           - the mean of active hidden nodes over all
%                          classifiers
%%%%    Authors:    JIAHUA LUO, CHI-MAN VONG
%%%%    UNIVERSITY OF MACAU, MACAO, CHINA
%%%%    EMAIL:      mb15457@umac.mo; cmvong@umac.mo
%%%%    WEBSITE:    http://www.fst.umac.mo/en/staff/fstcmv.html

function [Model,ActiveNodes] = BELM_Classification(X_train, T_train, label, number_class, NumberofHiddenNeurons,Activation,seed)
     if nargin <6
         Activation = 'sigmoid';
     end
     classifiersNo=0;
     ActiveNodes=0;
      for i=1: number_class
       for j=i+1: number_class
          Tr = [find(T_train==label(i))',find(T_train==label(j))'];                          %%
          t=T_train(Tr');                    
          X=X_train(Tr',:);
         % [t,X]          
          t(t==label(i))=0;  %setting the main class label as 1
          t(t==label(j))=1; 
          clear Temp;
          if nargin<7  % no seed provided
              [pairwisemodel]=PairwiseBelm(X,t,NumberofHiddenNeurons,Activation);
          else
             [pairwisemodel]=PairwiseBelm(X,t,NumberofHiddenNeurons,Activation,seed);
          end
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
          if isempty(pairwisemodel)
              Model=[];
              return;
          end
          classifiersNo = classifiersNo+1;
          Model(classifiersNo) = pairwisemodel;          
          clear X t Tr;
       end
      end
      ActiveNodes = NumberofHiddenNeurons;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%The input and output arguments are the same as SBELM's please refer to the m file Sbelm_Classify.m 

function [model]=PairwiseBelm(X,t,NumberofHiddenNeurons,Activation,seed)
% reference: BELM: Bayesian Extreme Learning Machine
if nargin<5
   %no seed for generating the hidden layer parameters provided
   [omega, InputWeights, HiddenBias]=Sbelm_Hiddenoutput(X,NumberofHiddenNeurons, Activation);
else 
    [omega, InputWeights, HiddenBias]=Sbelm_Hiddenoutput(X,NumberofHiddenNeurons, Activation,seed);
end
[N,M]=size(omega);
omega = [omega,ones(N,1)];
M=M+1;
alpha = 0;
W_ho = zeros(M,1);
new_alpha=0.1;
count=0;
%omega = omega+speye(N,M)/100;
while abs(new_alpha-alpha)>10e-8
  count=count+1;
  alpha = new_alpha;
  [W_ho,U] = BELM_PosteriorMode(omega,t,W_ho,alpha,200);
  diagSig	= trace(U);
  gamma = M-alpha*diagSig;
  new_alpha= (gamma)/(W_ho'*W_ho);
 % new_alpha= (gamma+2*1000)/(m'*m+2*100);
end
alpha
%%%%%%%%%%%%%%%%%%%%Prediction
clear y_t;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%save model
model=[];   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%save model
model.L=NumberofHiddenNeurons;
model.seed = seed;
model.hiddenfunc = 'sigmoid';
model.Wih= InputWeights;
model.hiddenBias = HiddenBias;
model.Who = W_ho;
model.usedHiddenbias = 'y'; 
end