% The approximation of gauss from Laplace, to find the center model of Gauss and covariance with 
% the Newton-Raphson method 'iterative reweighted least-squares (IRLS)',
% for theoretical inference. The difference between the SBELM and BELM model is that SBELM adopts indepandent ARD priors for the distribution of output weights
% while BELM just adopts one prior governing all weights distribution. pls refer to our paper "Sparse Bayesian Extreme Learning
% For more details, please refer to our paper Sparse Bayesian Extreme Learning Machine for Multi-classification by Jiahua Luo, Chi-Man Vong.etc
% This program is modified from the work of Micheal  Tipping(http://www.miketipping.com/sparsebayes.htm).
% Input arguments:
% PHI:       -the hidden output matrix of SBELM
% t:         -the expected output
% W         - the estimated weights from last iteraton
% Alpha     -the estimated Alpha from last iteration
% its       -the number of maximum iteration for IRLS
% output arguments:
% W         -the new updated w
% diagSigOfHessian:        -the diagonal matrix of inversed Hessian matrix
%                          for updating the new Alpha
% Ui:       - the inverse of hessian matrix
%%%%    Authors:    JIAHUA LUO, CHI-MAN VONG
%%%%    UNIVERSITY OF MACAU, MACAO, CHINA
%%%%    EMAIL:      mb15457@umac.mo; cmvong@umac.mo
%%%%    WEBSITE:    http://www.fst.umac.mo/en/staff/fstcmv.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [w,Ui] = BELM_PosteriorMode(PHI,t,w,alpha,its)
% This is the per-parameter gradient-norm threshold for termination
GRAD_STOP	= 1e-4;
% Limit to resolution of search step
LAMBDA_MIN	= 2^(-8);
[N d]	= size(PHI);
M 	= length(w);
A	= alpha;
errs	= zeros(its,1);
% PHI
PHIw	= PHI*w;
y	= sigmoid(PHIw);
t	= logical(t);
data_term	= -(sum(log(y(t))) + sum(log(1-y(~t))))/N;
regulariser	= (alpha*(norm(w)^2))/(2*N);
err_new		=  data_term + regulariser;

for i=1:its
  yvar	= y.*(1-y);
  PHIV	= PHI .* (yvar * ones(1,d));
  e	= (t-y);
  g		= PHI'*e - alpha*w;
  %PHI
  Hessian	= (PHIV'*PHI + A*speye(M));
  errs(i)	= err_new; 
  if i>=2 & norm(g)/M<GRAD_STOP
    break;
  end
  U		= inv(Hessian);% chol(Hessian);
  delta_w	= U* g;
  lambda	= 1;
  while lambda>LAMBDA_MIN
    w_new	= w + lambda*delta_w;
    PHIw	= PHI*w_new;
    y		= sigmoid(PHIw);
    % Compute new error 
    if any(y(t)==0) | any(y(~t)==1)
      err_new	= inf;
    else
      data_term		= -(sum(log(y(t))) + sum(log(1-y(~t))))/N;
      regulariser	= (alpha*(norm(w)^2))/(2*N);
      err_new		=  data_term + regulariser;
    end
    if err_new>errs(i)
      lambda	= lambda/2;
    else
      w		= w_new;
      lambda	= 0;
    end
  end
  % If we're here with non-zero lambda, then we couldn't take a small
  % enough downhill step: converged close to minimum
  if lambda
    break;
  end
end
% Compute requisite values at mode
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ui	= U;%inv(U); %U;%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Support function: sigmoid
function y = sigmoid(x)
y = 1./(1+exp(-x));

